/**
 * Copyright Openmind http://www.openmindonline.it
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package it.openutils.mgnltasks;

import info.magnolia.cms.core.SystemProperty;
import info.magnolia.module.DefaultModuleVersionHandler;
import info.magnolia.module.InstallContext;
import info.magnolia.module.delta.Delta;
import info.magnolia.module.delta.DeltaBuilder;
import info.magnolia.module.model.Version;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang.ObjectUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


/**
 * A base module version handler that re-bootstrap its configuration each time the version number changes. Can be used
 * as is or subclassed (usually overriding getStartupTasks() for configuration tweacks).
 * @author fgiust
 * @version $Id: SimpleModuleVersionHandler.java 803 2008-04-29 12:26:18Z fgiust $
 */
public class SimpleModuleVersionHandler extends DefaultModuleVersionHandler
{

    /**
     * Logger.
     */
    protected Logger log = LoggerFactory.getLogger(getClass());

    /**
     * {@inheritDoc}
     */
    @SuppressWarnings("unchecked")
    @Override
    public List<Delta> getDeltas(InstallContext ctx, Version from)
    {
        if (from == null)
        {
            log.info("Actual version not set");
            return super.getDeltas(ctx, from);
        }

        String modulename = ctx.getCurrentModuleDefinition().getName();

        Version to = ctx.getCurrentModuleDefinition().getVersionDefinition();

        List<Delta> deltas = new ArrayList<Delta>();

        if (SystemProperty.getBooleanProperty(modulename + ".update.disabled"))
        {
            return deltas;
        }

        if (!to.isEquivalent(from)
            || "${project.version}".equals(ObjectUtils.toString(from))
            || "SNAPSHOT".equals(from.getClassifier()))
        {
            log.info("Updating from version {}", from);

            Delta delta = DeltaBuilder.update(to, "Update to current version");
            delta.getTasks().add(new ModuleConfigBootstrapTask(modulename));
            delta.getTasks().add(new UpdateModuleVersionTask());

            deltas.add(delta);
        }
        else
        {
            log.info("Version {} already installed, no update tasks to run", from);
        }

        return deltas;
    }

}
